@@ -9,29 +9,29 @@
 #   Andy Wardley   <abw@kfs.org>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 1996-2006 Andy Wardley.  All Rights Reserved.
 #   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-#----------------------------------------------------------------------------
-#
-# $Id: Constants.pm,v 2.66 2004/01/12 12:37:54 abw Exp $
+# REVISION
+#   $Id: Constants.pm,v 2.71 2006/05/25 11:23:35 abw Exp $
 #
 #============================================================================
  
 package Template::Constants;
 
-require 5.004;
 require Exporter;
 
 use strict;
-use vars qw( $VERSION @ISA @EXPORT_OK %EXPORT_TAGS );
+use warnings;
+use base 'Exporter';
+
+use vars qw( @EXPORT_OK %EXPORT_TAGS );
 use vars qw( $DEBUG_OPTIONS @STATUS @ERROR @CHOMP @DEBUG);
 
-@ISA     = qw( Exporter );
-$VERSION = sprintf("%d.%02d", q$Revision: 2.66 $ =~ /(\d+)\.(\d+)/);
+our $VERSION = sprintf("%d.%02d", q$Revision: 2.71 $ =~ /(\d+)\.(\d+)/);
 
 
 #========================================================================
@@ -57,8 +57,10 @@ use constant ERROR_PLUGIN    =>  'plugin'; # plugin error
 
 # CHOMP constants for PRE_CHOMP and POST_CHOMP
 use constant CHOMP_NONE      => 0; # do not remove whitespace
-use constant CHOMP_ALL       => 1; # remove whitespace
+use constant CHOMP_ALL       => 1; # remove whitespace up to newline
+use constant CHOMP_ONE       => 1; # new name for CHOMP_ALL
 use constant CHOMP_COLLAPSE  => 2; # collapse whitespace to a single space
+use constant CHOMP_GREEDY    => 3; # remove all whitespace including newlines
 
 # DEBUG constants to enable various debugging options
 use constant DEBUG_OFF       =>    0; # do nothing
@@ -100,7 +102,7 @@ $DEBUG_OPTIONS  = {
                STATUS_DECLINED STATUS_ERROR );
 @ERROR   = qw( ERROR_FILE ERROR_VIEW ERROR_UNDEF ERROR_PERL 
                ERROR_RETURN ERROR_FILTER ERROR_PLUGIN );
-@CHOMP   = qw( CHOMP_NONE CHOMP_ALL CHOMP_COLLAPSE );
+@CHOMP   = qw( CHOMP_NONE CHOMP_ALL CHOMP_ONE CHOMP_COLLAPSE CHOMP_GREEDY );
 @DEBUG   = qw( DEBUG_OFF DEBUG_ON DEBUG_UNDEF DEBUG_VARS 
                DEBUG_DIRS DEBUG_STASH DEBUG_CONTEXT DEBUG_PARSER
                DEBUG_PROVIDER DEBUG_PLUGINS DEBUG_FILTERS DEBUG_SERVICE
@@ -228,8 +230,10 @@ The following tag sets and associated constants are defined:
 
   :chomp                      # for PRE_CHOMP and POST_CHOMP
     CHOMP_NONE                # do not remove whitespace
-    CHOMP_ALL                 # remove whitespace
+    CHOMP_ONE                 # remove whitespace to newline
+    CHOMP_ALL                 # old name for CHOMP_ONE (deprecated)
     CHOMP_COLLAPSE            # collapse whitespace to a single space
+    CHOMP_GREEDY              # remove all whitespace including newlines
 
   :debug
     DEBUG_OFF                 # do nothing
@@ -252,21 +256,21 @@ The following tag sets and associated constants are defined:
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@andywardley.comE<gt>
+Andy Wardley E<lt>abw@wardley.orgE<gt>
 
-L<http://www.andywardley.com/|http://www.andywardley.com/>
+L<http://wardley.org/|http://wardley.org/>
 
 
 
 
 =head1 VERSION
 
-2.65, distributed as part of the
-Template Toolkit version 2.12, released on 12 January 2004.
+2.71, distributed as part of the
+Template Toolkit version 2.15, released on 26 May 2006.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1996-2006 Andy Wardley.  All Rights Reserved.
   Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
